using System.Collections.Generic;
using UnityEngine;
using System;
using Battlehub.RTSaveLoad;

namespace Ruvik
{
    class PH : MVRScript
    {
        JSONStorableStringChooser atomJSON;
        UIDynamicPopup atomPopup;
        static Keybind activationKb, switchMotionKb, switchAxisKb, UIToggleKb,clearSelectionKb;
        Dictionary<string, Keybind> keyBindPairs = new Dictionary<string, Keybind>();
        static bool isRecording;
        string keyString = string.Empty;
        public static JSONStorableFloat sensetivity, rotationSpeed;
        public override void Init()
        {
            PHUI.InitPH();
            CreateAtomChooser();
            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

            sensetivity = new JSONStorableFloat("sensetivity", 1.0f, 0.1f, 4.0f);
            rotationSpeed = new JSONStorableFloat("rotation Speed", 1.0f, 0.1f, 10.0f);

            activationKb = GenKey("start the motion", KeyCode.Space);
            switchMotionKb = GenKey("switch the motion(position to rotation)", KeyCode.LeftShift);
            switchAxisKb = GenKey("switch the axis", KeyCode.LeftControl);
            UIToggleKb = GenKey("Activate the UI", KeyCode.L);
            clearSelectionKb = GenKey("Clear selected bones", KeyCode.Backspace);

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

            CreateSlider(sensetivity, false);
            CreateSlider(rotationSpeed, true);

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

            CreateToggle(new JSONStorableBool("invert X", false),false).toggle.onValueChanged.AddListener(delegate (bool a) { PHUtils.xMultiplier = a?-1.0f:1.0f; });
            CreateToggle(new JSONStorableBool("invert Z", false),true).toggle.onValueChanged.AddListener(delegate (bool a) { PHUtils.zMultiplier = a?-1.0f:1.0f; });
        }
        public Keybind GenKey(string actionName,KeyCode defaultKey)
        {
            CreateButton(actionName, false);
            var btn = CreateButton(defaultKey.ToString(), true);
            Keybind kb = new Keybind(defaultKey,btn);
            keyBindPairs.Add(actionName, kb);
            btn.button.onClick.AddListener(delegate () { keyString = actionName; btn.label = "....";btn.buttonColor = Color.red;isRecording = true; });
            return kb;
        }
        public void FetchKeyOrButton(Keybind kb)
        {
            foreach (KeyCode kcode in Enum.GetValues(typeof(KeyCode)))
            {
                if (Input.GetKeyDown(kcode))
                {
                    kb.Update(kcode);
                }
            }
        }
        public class Keybind
        {
            public KeyCode key;
            UIDynamicButton UIbtn;
            public Keybind(KeyCode defaultKey, UIDynamicButton btn)
            {
                key = defaultKey;
                UIbtn = btn;
            }
            public void Update(KeyCode key)
            {
                isRecording = false;
                UIbtn.buttonColor = new Color(0.78f, 0.78f, 0.78f);
                this.UIbtn.label = key.ToString();
                this.key = key;
            }
        }

        private void CreateAtomChooser()
        {
            atomJSON = new JSONStorableStringChooser("atom", SuperController.singleton.GetAtomUIDs(), null, "Atom");
            RegisterStringChooser(atomJSON);
            atomPopup = CreateFilterablePopup(atomJSON);
            var addBtn = CreateButton("Add",true);
            addBtn.height = atomPopup.height;
            addBtn.button.onClick.AddListener(delegate ()
            {
                if(atomJSON.val != null && atomJSON.val != string.Empty)
                PHUI.AddControableAtom(atomPopup.popup.currentValue);
            });
            atomPopup.popup.onOpenPopupHandlers += SyncAtomChocies;
        }
        private void CheckForKeybindUpdate()
        {
            if(isRecording)
                FetchKeyOrButton(keyBindPairs.Get(keyString));

        }
        private void ApplySymmetry()
        {
            if (PHUtils.isSymmetryToggled)
            {
                PHUtils.MirrorArmsMethod(false);
                PHUtils.MirrorLegsMethod(false);
            }

        }
        void Update()
        {
            if (Input.GetKeyDown(UIToggleKb.key))
            {
                PHUI.PHMenuSetActive(!PHUI.phGameObject.activeInHierarchy);
            }

            if (Input.GetKeyDown(clearSelectionKb.key))
                PHUtils.ClearPressedButtons();

            if(Input.GetKeyDown(activationKb.key))
                PHUtils.PreMoveControllers();

            else if(Input.GetKey(activationKb.key))
            {
                float x, y;
                Vector3 moveby;
                x = Input.GetAxis("Mouse X");
                y = Input.GetAxis("Mouse Y");
                moveby = Input.GetKey(switchAxisKb.key) ? new Vector3(0, 0, x + y) : new Vector3(x, y, 0);
                PHUtils.MoveActiveControllers(moveby,!Input.GetKey(switchMotionKb.key));
            }
            if (Input.GetKeyUp(activationKb.key))
                PHUtils.PostMoveControllers();

            ApplySymmetry();
            CheckForKeybindUpdate();
        }
        public void OnDestroy()
        {
            PHUI.PHMenuDesroy();
        }
        public void OnDisable()
        {
            PHUI.PHMenuSetActive(false);
        }
        public void OnEnable()
        {
            PHUI.PHMenuSetActive(true);
        }
        protected void SyncAtomChocies()
        {
            List<string> atomChoices = new List<string>();
            atomChoices.Add(string.Empty);
            foreach (string atomUID in SuperController.singleton.GetAtomUIDsWithFreeControllers())
            {
                if (SuperController.singleton.GetAtomByUid(atomUID).freeControllers.Length > 10)
                    atomChoices.Add(atomUID);
            }
            atomJSON.choices = atomChoices;
        }
    }
}
