using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

namespace Ruvik
{
    public class PHUI
    {
        public static GameObject phGameObject = null;
        static Sprite PHButtonSprite = null;
        static List<string> controlableAtomsNames = null;
        static Text controlledAtomNameTextUI = null;
        private static int currentNameIndex = 0;
        static public bool isLocalToggled = true;
        static public void InitPH()
        {
            if(GameObject.Find("posing helper canvas")!=null)
            {
                SuperController.LogError("please make sure there's only one instance of this plugin in the scene");
                return;
            }
            PHButtonSprite = GameObject.Find("MainWindowUI").GetComponentInChildren<Image>().sprite;
            phGameObject = new GameObject("posing helper canvas");
            var phCanvas = phGameObject.AddComponent<Canvas>();
            phCanvas.renderMode = RenderMode.ScreenSpaceOverlay;
            phGameObject.AddComponent<CanvasScaler>().uiScaleMode = CanvasScaler.ScaleMode.ScaleWithScreenSize;
            phGameObject.AddComponent<GraphicRaycaster>();
            phGameObject.GetComponent<CanvasScaler>().screenMatchMode = CanvasScaler.ScreenMatchMode.Expand;
            InitPHMenu();
            CreateMenus();
            CreateButtons();

            controlledAtomNameTextUI = GetPHButton("name").GetComponentInChildren<Text>();
            controlledAtomNameTextUI.text = "None";
            GetPHButton("next").onClick.AddListener(delegate () { SelectAtom(true); });
            GetPHButton("prev").onClick.AddListener(delegate () { SelectAtom(false); });
			GetPHButton("PX").gameObject.AddComponent<MovementButtons>();
			GetPHButton("PY").gameObject.AddComponent<MovementButtons>();
			GetPHButton("PZ").gameObject.AddComponent<MovementButtons>();
			GetPHButton("RX").gameObject.AddComponent<MovementButtons>();
			GetPHButton("RY").gameObject.AddComponent<MovementButtons>();
			GetPHButton("RZ").gameObject.AddComponent<MovementButtons>();
			GetPHButton("State").onClick.AddListener(LocalButtonAction());
			GetPHButton("Symmetry").onClick.AddListener(SymmetryButtonAction());
            GetPHButton("Reset").onClick.AddListener(PHUtils.Reset());
            GetPHButton("Link").onClick.AddListener(PHUtils.LinkAll());
            GetPHButton("MRA").onClick.AddListener(PHUtils.MirrorArmsAction(false));
            GetPHButton("MLA").onClick.AddListener(PHUtils.MirrorArmsAction(true));
            GetPHButton("MRL").onClick.AddListener(PHUtils.MirrorLegsAction(false));
            GetPHButton("MLL").onClick.AddListener(PHUtils.MirrorLegsAction(true));
            GetPHButton("SA").onClick.AddListener(PHUtils.SwapArms());
            GetPHButton("SL").onClick.AddListener(PHUtils.SwapLegs());
            SuperController.singleton.AddCanvas(phCanvas);
            phGameObject.SetActive(false);
        }

        static public UnityAction LocalButtonAction()
        {
            return delegate
            {
                PHUtils.SwitchMovementMode();
            };
        }

        static public UnityAction SymmetryButtonAction()
        {
            return delegate
            {
                PHUtils.ToggleSymmetry();
            };
        }
        static public Button GetPHButton(string name)
        {
            GameObject phButtonGo = GameObject.Find("PH_" + name);

            if (phButtonGo == null)
                SuperController.LogError("wrong button name");
            else
                return phButtonGo.GetComponent<Button>();

            return null;
        }
        static public void AddControableAtom(string name)
        {
            if (!phGameObject.activeInHierarchy)
                phGameObject.SetActive(true);
            if (controlableAtomsNames == null)
                controlableAtomsNames = new List<string> { name };
            else
                if (!controlableAtomsNames.Contains(name))
                    controlableAtomsNames.Add(name);
            SelectAtom(true);
        }
        static private void SelectAtom(bool next)
        {
            if (controlableAtomsNames.Count == 0)
                return;
            string atomName;

            if (next)
                currentNameIndex = currentNameIndex == controlableAtomsNames.Count -1 ? 0 : currentNameIndex + 1;
            else
                currentNameIndex = currentNameIndex == 0 ? controlableAtomsNames.Count -1 : currentNameIndex - 1;

            atomName = controlableAtomsNames.ElementAt(currentNameIndex);
            if (atomName != null)
            {
                controlledAtomNameTextUI.text = atomName;
                PHUtils.InitPHControllers(SuperController.singleton.GetAtomByUid(atomName));
            }

        }
        static public void PHMenuDesroy()
        {
            if(phGameObject!=null)
            GameObject.Destroy(phGameObject);
        }
        static public void PHMenuSetActive(bool state)
        {
            if(phGameObject!=null)
            phGameObject.SetActive(state);
        }
        static private void InitPHMenu()
        {
            foreach (var item in UIRectSpecs)
            {
                GameObject go = new GameObject(GetName(item));

                GameObject root = GameObject.Find(GetParent(item));
                go.transform.SetParent(root.transform);

                var rootRect = go.AddComponent<RectTransform>();
                SetRect(ref rootRect, GetData(item));
            }
        }
        static private void CreateButtons()
        {
            foreach (var item in UIButtonSpecs)
            {
                GameObject go = GameObject.Find(GetName(item));
                go.AddComponent<Button>();
                var img = go.GetComponent<Image>();
                img.sprite = PHButtonSprite;
                img.type = Image.Type.Sliced;
                img.type = Image.Type.Sliced;
                GameObject textGo = new GameObject("PH_text");
                textGo.transform.SetParent(go.transform);
                var textRect = textGo.AddComponent<RectTransform>();
                var text = textGo.AddComponent<Text>();
                SetRect(ref textRect, UITextRectSpecs);
                text.text = GetData(item);
                text.fontSize = 12;
                text.fontStyle = FontStyle.Normal;
                text.color = Color.black;
                text.alignment = TextAnchor.MiddleCenter;
                text.font = Resources.GetBuiltinResource<Font>("Arial.ttf");
            }
        }
        static private void CreateMenus()
        {
            foreach (var item in UIImgSpecs)
            {
                var rootImage = GameObject.Find(GetName(item)).AddComponent<Image>();
                rootImage.color = GenColor(GetData(item));
            }
        }

        static private string[] UIRectSpecs = new string[]
        {
        "posing helper canvas/PH_menu:-10,80,1,0,150,230,1,0,1,0,-160,80,-10,310",
        "PH_menu/PH_bones:0,10,0.5,0.5,0,-20,0,0,1,1,0,20,0,0",
        "PH_bones/PH_head:0,88,0.5,0.5,30,20,0.5,0.5,0.5,0.5,-15,78,15,98",
        "PH_bones/PH_R.shoulder:40,72,0.5,0.5,50,20,0.5,0.5,0.5,0.5,15,62,65,82",
        "PH_bones/PH_L.shoulder:-40,72,0.5,0.5,50,20,0.5,0.5,0.5,0.5,-65,62,-15,82",
        "PH_bones/PH_L.elbow:-50,50,0.5,0.5,40,20,0.5,0.5,0.5,0.5,-70,40,-30,60",
        "PH_bones/PH_R.elbow:50,50,0.5,0.5,40,20,0.5,0.5,0.5,0.5,30,40,70,60",
        "PH_bones/PH_L.hand:-55,25,0.5,0.5,40,20,0.5,0.5,0.5,0.5,-75,15,-35,35",
        "PH_bones/PH_R.hand:55,25,0.5,0.5,40,20,0.5,0.5,0.5,0.5,35,15,75,35",
        "PH_bones/PH_chest:0,50,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-30,40,30,60",
        "PH_bones/PH_waist:0,27,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-30,17,30,37",
        "PH_bones/PH_pelvis:0,5,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-30,-5,30,15",
        "PH_bones/PH_hip:0,-18,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-30,-28,30,-8",
        "PH_bones/PH_L.thigh:-30,-40,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-60,-50,0,-30",
        "PH_bones/PH_R.thigh:30,-40,0.5,0.5,60,20,0.5,0.5,0.5,0.5,0,-50,60,-30",
        "PH_bones/PH_L.knee:-35,-65,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-65,-75,-5,-55",
        "PH_bones/PH_R.knee:35,-65,0.5,0.5,60,20,0.5,0.5,0.5,0.5,5,-75,65,-55",
        "PH_bones/PH_L.foot:-45,-90,0.5,0.5,60,20,0.5,0.5,0.5,0.5,-75,-100,-15,-80",
        "PH_bones/PH_R.foot:45,-90,0.5,0.5,60,20,0.5,0.5,0.5,0.5,15,-100,75,-80",
        "PH_menu/PH_select:0,-105,0.5,0.5,0,-210,0,0,1,1,0,0,0,-210",
        "PH_select/PH_prev:5,0,0,0.5,20,0,0,0,0,1,5,0,25,0",
        "PH_select/PH_name:0,0,0.5,0.5,90,0,0.5,0,0.5,1,-45,0,45,0",
        "PH_select/PH_next:-5,0,1,0.5,20,0,1,0,1,1,-24,0,-5,1",
        "posing helper canvas/PH_movement:-10,10,1,0,150,70,1,0,1,0,-160,10,-10,80",
        "PH_movement/PH_PZ:6.103516E-05,0,1,1,50,36,1,1,1,1,-49.99994,-36,6.103516E-05,0",
        "PH_movement/PH_PY:6.103516E-05,0,0.5,1,50,36,0.5,1,0.5,1,-24.99994,-36,25.00006,0",
        "PH_movement/PH_PX:6.103516E-05,0,0,1,50,36,0,1,0,1,6.103516E-05,-36,50.00006,0",
        "PH_movement/PH_RZ:6.103516E-05,0,1,0,50,36,1,0,1,0,-49.99994,0,6.103516E-05,36",
        "PH_movement/PH_RY:6.103516E-05,0,0.5,0,50,36,0.5,0,0.5,0,-24.99994,0,25.00006,36",
        "PH_movement/PH_RX:6.103516E-05,0,0,0,50,36,0,0,0,0,6.103516E-05,0,50.00006,36",
        "posing helper canvas/PH_functions:-160,10,1,0,80,300,1,0,1,0,-240,10,-160,310",
        "PH_functions/PH_MRA:0,0,0.5,1,0,25.52547,0,1,1,1,0,-25.52547,0,0",
        "PH_functions/PH_MLA:0,-25.89,0.5,1,0,25.52547,0,1,1,1,0,-51.41547,0,-25.89",
        "PH_functions/PH_MRL:0,-55.66002,0.5,1,0,25.52547,0,1,1,1,0,-81.18549,0,-55.66002",
        "PH_functions/PH_MLL:0,-81.55,0.5,1,0,25.52547,0,1,1,1,0,-107.0755,0,-81.55",
        "PH_functions/PH_SA:0,-111.33,0.5,1,0,25.52547,0,1,1,1,0,-136.8555,0,-111.33",
        "PH_functions/PH_SL:0,-137.21,0.5,1,0,25.52547,0,1,1,1,0,-162.7355,0,-137.21",
        "PH_functions/PH_Symmetry:0,-164.19,0.5,1,0,32.74202,0,1,1,1,0,-196.932,0,-164.19",
        "PH_functions/PH_State:0,-199.19,0.5,1,0,32.74202,0,1,1,1,0,-231.932,0,-199.19",
        "PH_functions/PH_Link:0,-234.19,0.5,1,0,32.74202,0,1,1,1,0,-266.932,0,-234.19",
        "PH_functions/PH_Reset:0,-267.39,0.5,1,0,32.74202,0,1,1,1,0,-300.132,0,-267.39",
        };
        static private string[] UIImgSpecs = new string[]
        {
        "PH_menu:0.2,0.3,0.4,0.0",
        "PH_bones:0.2,0.3,0.4,0.3",
        "PH_head:1,1,1,1",
        "PH_R.shoulder:1,1,1,1",
        "PH_L.shoulder:1,1,1,1",
        "PH_L.elbow:1,1,1,1",
        "PH_R.elbow:1,1,1,1",
        "PH_L.hand:1,1,1,1",
        "PH_R.hand:1,1,1,1",
        "PH_chest:1,1,1,1",
        "PH_waist:1,1,1,1",
        "PH_pelvis:1,1,1,1",
        "PH_hip:1,1,1,1",
        "PH_L.thigh:1,1,1,1",
        "PH_R.thigh:1,1,1,1",
        "PH_L.knee:1,1,1,1",
        "PH_R.knee:1,1,1,1",
        "PH_L.foot:1,1,1,1",
        "PH_R.foot:1,1,1,1",
        "PH_select:0.2,0.3,0.4,0.3",
        "PH_prev:1,1,1,1",
        "PH_name:1,1,1,1",
        "PH_next:1,1,1,1",
        "PH_movement:0.2,0.3,0.4,0.3",
        "PH_PZ:0,0,1,1",
        "PH_PY:0,1,0,1",
        "PH_PX:1,0,0,1",
        "PH_RZ:0,0.5,1,1",
        "PH_RY:0.5,1,0,1",
        "PH_RX:1,0,0.5,1",
        "PH_functions:0.2,0.3,0.4,0.3",
        "PH_MRA:1,1,1,1",
        "PH_MLA:1,1,1,1",
        "PH_MRL:1,1,1,1",
        "PH_MLL:1,1,1,1",
        "PH_SA:1,1,1,1",
        "PH_SL:1,1,1,1",
        "PH_Symmetry:1,1,1,1",
        "PH_State:1,1,1,1",
        "PH_Link:1,1,1,1",
        "PH_Reset:1,1,1,1",
        };
        static private string[] UIButtonSpecs = new string[]
        {
        "PH_head:head",
        "PH_R.shoulder:R.shoulder",
        "PH_L.shoulder:L.shoulder",
        "PH_L.elbow:L.elbow",
        "PH_R.elbow:R.elbow",
        "PH_L.hand:L.hand",
        "PH_R.hand:R.hand",
        "PH_chest:chest",
        "PH_waist:waist",
        "PH_pelvis:pelvis",
        "PH_hip:Hip",
        "PH_L.thigh:L.thigh",
        "PH_R.thigh:R.thigh",
        "PH_L.knee:L.knee",
        "PH_R.knee:R.knee",
        "PH_L.foot:L.foot",
        "PH_R.foot:R.foot",
        "PH_prev:<<",
        "PH_name:Button",
        "PH_next:>>",
        "PH_PZ:Z",
        "PH_PY:Y",
        "PH_PX:X",
        "PH_RZ:<-Z->",
        "PH_RY:<-Y->",
        "PH_RX:<-X->",
        "PH_MRA:Copy L.Arm",
        "PH_MLA:Copy R.Arm",
        "PH_MRL:Copy L.Leg",
        "PH_MLL:Copy R.Leg",
        "PH_SA:Swap Arms",
        "PH_SL:Swap Legs",
        "PH_Symmetry:Symmetry",
        "PH_State:Local",
        "PH_Link:Link",
        "PH_Reset:Reset",
        };
        static private string UITextRectSpecs = "0,0,0.5,0.5,0,0,0,0,1,1,0,0,0,0";
        static Color GenColor(string specs)
        {
            float[] fArray = GetArrayFromString(specs);
            return new Color(fArray[0], fArray[1], fArray[2], fArray[3]);
        }
        static private void SetRect(ref RectTransform rt, string specs)
        {
            float[] fArray = GetArrayFromString(specs);
            rt.anchoredPosition3D = new Vector3(fArray[0], fArray[1], 0);
            rt.pivot = new Vector2(fArray[2], fArray[3]);
            rt.sizeDelta = new Vector2(fArray[4], fArray[5]);
            rt.anchorMin = new Vector2(fArray[6], fArray[7]);
            rt.anchorMax = new Vector2(fArray[8], fArray[9]);
            rt.offsetMin = new Vector2(fArray[10], fArray[11]);
            rt.offsetMax = new Vector2(fArray[12], fArray[13]);
            rt.localEulerAngles = Vector3.zero;
            rt.localScale = Vector3.one;
        }
        static private float[] GetArrayFromString(string str)
        {
            return Array.ConvertAll(str.Split(','), float.Parse);
        }

        static private string GetName(string specs)
        {
            string name = "";
            int i;
            if (specs.Contains("/"))
                i = specs.IndexOf('/') + 1;
            else
                i = 0;

            for (; i < specs.Length; i++)
            {
                if (specs[i] == ':')
                    break;
                name += specs[i];
            }
            return name;
        }
        static private string GetParent(string specs)
        {
            string name = "";
            for (int i = 0; i < specs.Length; i++)
            {
                if (specs[i] == '/')
                    break;
                name += specs[i];
            }
            return name;
        }
        static private string GetData(string specs)
        {
            return specs.Substring(specs.IndexOf(':') + 1);
        }
    }
}